@@ -7,7 +7,7 @@ require Exporter;
 require DynaLoader;
 @ISA = qw(Exporter DynaLoader);
 
-$VERSION = "0.271";
+$VERSION = "0.30";
 
 bootstrap XML::Bare $VERSION;
 
@@ -20,7 +20,7 @@ XML::Bare - Minimal XML parser implemented via a C state engine
 
 =head1 VERSION
 
-0.271 including simple-like tree building
+0.30
 
 =cut
 
@@ -30,8 +30,6 @@ sub new {
   my $self  = {};
   %$self    = @_;
 
-  $self->{forcearray} = +{ map { $_ => 1 } @{$self->{forcearray} || []} } if(ref $self->{forcearray} ne 'HASH');
-
   bless $self, $class;
   
   my $text;
@@ -74,7 +72,7 @@ sub merge {
     my $short = $hash{ $one->{ $id }->{ 'value' } };
     next if( !$short );
     foreach my $key ( keys %$one ) {
-      next if( $key eq 'pos' || $key eq 'id' );
+      next if( $key eq '_pos' || $key eq 'id' );
       my $cur = $short->{ $key };
       my $add = $one->{ $key };
       if( !$cur ) {
@@ -123,11 +121,10 @@ sub parse {
   return $self->{ 'xml' };
 }
 
-# Load a file using XML::DOM, convert it to a hash, and return the hash
 sub simple {
   my $self   = shift;
   
-  $self->{ 'xml' } = XML::Bare::xml2obj_simple($self->{forcearray});#$self->xml2obj();
+  $self->{ 'xml' } = XML::Bare::xml2obj_simple();#$self->xml2obj();
   XML::Bare::free_tree();
   
   return $self->{ 'xml' };
@@ -146,6 +143,33 @@ sub add_node {
   return $newnode;
 }
 
+sub add_node_after {
+  my $self = shift;
+  my $node = shift;
+  my $prev = shift;
+  my $name = shift;
+  my @newar;
+  my %blank;
+  $node->{ 'multi_'.$name } = \%blank if( ! $node->{ 'multi_'.$name } );
+  $node->{ $name } = \@newar if( ! $node->{ $name } );
+  my $newnode = $self->new_node( @_ );
+  
+  my $cur = 0;
+  for my $anode ( @{ $node->{ $name } } ) {
+    $anode->{'_pos'} = $cur if( !$anode->{'_pos'} );
+    $cur++;
+  }
+  my $opos = $prev->{'_pos'};
+  for my $anode ( @{ $node->{ $name } } ) {
+    $anode->{'_pos'}++ if( $anode->{'_pos'} > $opos );
+  }
+  $newnode->{'_pos'} = $opos + 1;
+  
+  push( @{ $node->{ $name } }, $newnode );
+  
+  return $newnode;
+}
+
 sub find_by_perl {
   my $arr = shift;
   my $cond = shift;
@@ -270,7 +294,9 @@ sub xml {
   my $obj = shift;
   my $name = shift;
   if( !$name ) {
-    return obj2xml( $obj, '', 0 );
+    my %hash;
+    $hash{0} = $obj;
+    return obj2xml( \%hash, '', 0 );
   }
   my %hash;
   $hash{$name} = $obj;
@@ -282,6 +308,7 @@ sub obj2xml {
   my $name = shift;
   my $pad = shift;
   my $level = shift;
+  my $pdex = shift;
   $level = 0 if( !$level );
   $pad = '' if( $level == 1 );
   my $xml  = '';
@@ -294,12 +321,14 @@ sub obj2xml {
       my $obb = $objs->{ $b };
       if( !$oba ) { return 0; }
       if( !$obb ) { return 0; }
+      $oba = $oba->[0] if( ref( $oba ) eq 'ARRAY' );
+      $obb = $obb->[0] if( ref( $obb ) eq 'ARRAY' );
       if( ref( $oba ) eq 'HASH' && ref( $obb ) eq 'HASH' ) {
-        my $posa = $oba->{'pos'};
-        my $posb = $obb->{'pos'};
+        my $posa = $oba->{'_pos'}*1;
+        my $posb = $obb->{'_pos'}*1;
         if( !$posa ) { $posa = 0; }
         if( !$posb ) { $posb = 0; }
-        return $posa cmp $posb;
+        return $posa <=> $posb;
       }
       return 0;
     } keys %$objs;
@@ -308,8 +337,27 @@ sub obj2xml {
     my $type = ref( $obj );
     if( $type eq 'ARRAY' ) {
       $imm = 0;
-      for( my $j = 0; $j <= $#$obj; $j++ ) {
-        $xml .= obj2xml( $obj->[ $j ], $i, $pad.'  ', $level+1 );
+      
+      my @dex2 = sort
+        { 
+          my $oba = $a;#$obj->[ $a ];
+          my $obb = $b;#$obj->[ $b ];
+          if( !$oba ) { return 0; }
+          if( !$obb ) { return 0; }
+          if( ref( $oba ) eq 'HASH' && ref( $obb ) eq 'HASH' ) {
+            my $posa = $oba->{'_pos'};
+            my $posb = $obb->{'_pos'};
+            if( !$posa ) { $posa = 0; }
+            if( !$posb ) { $posb = 0; }
+            return $posa <=> $posb;
+          }
+          return 0;
+        } @$obj;
+      
+      #for( my $j = 0; $j <= $#$obj; $j++ ) {
+      for my $j ( @dex2 ) {
+        #$xml .= obj2xml( $obj->[ $j ], $i, $pad.'  ', $level+1, $#dex );
+        $xml .= obj2xml( $j, $i, $pad.'  ', $level+1, $#dex );
       }
     }
     elsif( $type eq 'HASH' ) {
@@ -318,7 +366,7 @@ sub obj2xml {
         $att .= ' ' . $i . '="' . $obj->{ 'value' } . '"';
       }
       else {
-        $xml .= obj2xml( $obj , $i, $pad.'  ', $level+1 );
+        $xml .= obj2xml( $obj , $i, ( $level > 1 ) ? ( $pad.'  ' ) : '', $level+1, $#dex );
       }
     }
     else {
@@ -331,12 +379,21 @@ sub obj2xml {
             $xml .= '<![CDATA[' . $obj . ']]>';
           }
           else {
-            $xml .= $obj;
+            if( $obj =~ /\S/ ) {
+              #if( $#dex == 1 ) {
+              #  $obj =~ s/^\s+//;
+              #  $obj =~ s/\s+$//;
+              #  $obj = "$pad  $obj\n";
+              #}
+              $xml .= $obj;
+            }
           }
         }
       }
+      elsif( substr( $i, 0, 1 ) eq '_' ) {
+      }
       else {
-        $xml .= '<' . $i . '>' . $obj . '</' . $i . '>' if( $i ne 'pos' );
+        $xml .= '<' . $i . '>' . $obj . '</' . $i . '>';
       }
     }
   }
@@ -359,17 +416,13 @@ __END__
 
   use XML::Bare;
   
-  my $xml = new XML::Bare( text => '<xml><name>Bob</name></xml>', forcearray => [qw/tag1 tag2/] );
+  my $xml = new XML::Bare( text => '<xml><name>Bob</name></xml>' );
   
   # Parse the xml into a hash tree
   my $root = $xml->parse();
   
-  # Parse the xml into a L<XML::Simple>-like hash tree using forcearray option from C<new> method.
-  my $simple_root = $xml->simple();
-  
   # Print the content of the name node
   print $root->{xml}->{name}->{value};
-  print $simple_root->{xml}->{name};
   
   # Load xml from a file ( assume same contents as first example )
   my $xml2 = new XML::Bare( file => 'test.xml' );
@@ -400,51 +453,41 @@ examples. Each of the PERL statements evaluates to true.
 
   XML: <xml>blah</xml>
   PERL: $root->{xml}->{value} eq "blah";
-  PERL: $simple_root->{xml} eq "blah";
 
 =item * Subset nodes
 
   XML: <xml><name>Bob</name></xml>
   PERL: $root->{xml}->{name}->{value} eq "Bob";
-  PERL: $simple_root->{xml}->{name} eq "Bob";
 
 =item * Attributes unquoted
 
   XML: <xml><a href=index.htm>Link</a></xml>
   PERL: $root->{xml}->{a}->{href}->{value} eq "index.htm";
-  PERL: $simple_root->{xml}->{a}->{href} eq "index.htm";
 
 =item * Attributes quoted
 
   XML: <xml><a href="index.htm">Link</a></xml>
   PERL: $root->{xml}->{a}->{href}->{value} eq "index.htm";
-  PERL: $simple_root->{xml}->{a}->{href} eq "index.htm";
 
 =item * CDATA nodes
 
   XML: <xml><raw><![CDATA[some raw $~<!bad xml<>]]></raw></xml>
   PERL: $root->{xml}->{raw}->{value} eq "some raw \$~<!bad xml<>";
-  PERL: $simple_root->{xml}->{raw} eq "some raw \$~<!bad xml<>";
 
 =item * Multiple nodes; form array
 
   XML: <xml><item>1</item><item>2</item></xml>
   PERL: $root->{xml}->{item}->[0]->{value} eq "1";
-  PERL: $simple_root->{xml}->{item}->[0] eq "1";
 
 =item * Forcing array creation
 
-  XML: <xml><multi_item/><item>i1</item><tag1>t1</tag1><tag2>t2</tag2></xml>
-  PERL: $root->{xml}->{item}->[0]->{value} eq "i1";
-  PERL: $simple_root->{xml}->{item} eq "i1";
-  PERL: $simple_root->{xml}->{tag1}->[0] eq "t1";
-  PERL: $simple_root->{xml}->{tag2}->[0] eq "t2";
+  XML: <xml><multi_item/><item>1</item></xml>
+  PERL: $root->{xml}->{item}->[0]->{value} eq "1";
 
 =item * One comment supported per node
 
   XML: <xml><!--test--></xml>
   PERL: $root->{xml}->{comment} eq 'test';
-  PERL: $simple_root has no comments
 
 =back
 
@@ -455,11 +498,11 @@ Besides as described above, the structure contains some additional nodes in
 order to preserve information that will allow that structure to be correctly
 converted back to XML.
   
-Nodes may contain the following 2 additional subnodes (not $simple_root):
+Nodes may contain the following 2 additional subnodes:
 
 =over 2
 
-=item * pos
+=item * _pos
 
 This is a number indicating the ordering of nodes. It is used to allow
 items in a perl hash to be sorted when writing back to xml. Note that
@@ -493,13 +536,16 @@ is output as CDATA.
 
 =item * Node position stored, but hash remains unsorted
 
-The ordering of nodes is noted using the 'pos' value, but
+The ordering of nodes is noted using the '_pos' value, but
 the hash itself is not ordered after parsing. Currently
 items will be out of order when looking at them in the
 hash.
 
 Note that when converted back to XML, the nodes are then
-sorted and output in the correct order to XML.
+sorted and output in the correct order to XML. Note that
+nodes of the same name with the same parent will be
+grouped together; the position of the first item to
+appear will determine the output position of the group.
 
 =item * Comments are parsed but only one is stored per node.
 
@@ -517,18 +563,14 @@ immediate values when a node contains no subnodes.
 
 =item * Unknown C<< <! >> sections are parsed, but discarded
 
-=item * Attributes must use double quotes if quoted
-
-Attributes in XML can be used, with or without quotes, but
-if quotes are used they must be double quotes. If single
-quotes are used, the value will end up starting with a single
-quote and continue until a space or a node end.
+=item * Attributes may use no quotes, single quotes, quotes
 
 =item * Quoted attributes cannot contain escaped quotes
 
 No escape character is recognized within quotes. As a result,
-there is no way to store a double quote character in an
-attribute value.
+regular quotes cannot be stored to XML, or the written XML
+will not be correct, due to all attributes always being written
+using quotes.
 
 =item * Attributes are always written back to XML with quotes
 
@@ -555,7 +597,7 @@ equal to the first continuous string of text besides a subnode.
 
 =over 2
 
-=item * C<< $ob = new XML::Bare( text => "[some xml]", forcearray => [qw/tag1 tag2/] ) >>
+=item * C<< $ob = new XML::Bare( text => "[some xml]" ) >>
 
 Create a new XML object, with the given text as the xml source.
 
@@ -574,8 +616,18 @@ Parse the xml of the object and return a tree reference
 
 =item * C<< $tree = $object->simple() >>
 
-Parse the xml of the object and return a L<XML::Simple>-like tree reference.
-No L<XML::Simple> features available, excepting I<forcearray>
+Alternate to the parse function which generates a tree similar to that
+generated by XML::Simple. Note that the sets of nodes are turned into
+arrays always, regardless of whether they have a 'name' attribute, unlike
+XML::Simple.
+
+Note that currently the generated tree cannot be used with any of the
+functions in this module that operate upon trees. The function is provided
+purely as a quick and dirty way to read simple XML files.
+
+Also note that you cannot rely upon this function being contained in
+future versions of XML::Bare; the function will likely be split off into
+an optimized version meant purely to operate in this fashion.
 
 =item * C<< $text = $object->xml( [root] ) >>
 
@@ -618,6 +670,10 @@ Clean up the xml in filename1 and save the results to filename2.
         <name>Bob</name>
       </item>
     </xml>
+    
+=item * C<< $oject->add_node_after( [node], [prev node], [nodeset name], name => value, name2 => value2, ... ) >>
+
+Similar to add_node above, but adds the node immediately after the passed [prev node].
 
 =item * C<< $object->del_node( [node], [nodeset name], name => value ) >>
 
@@ -755,6 +811,46 @@ Example:
       </a>
     </xml>
 
+=item * C<< XML::Bare::del_by_perl( ... ) >>
+
+Works exactly like find_by_perl, but deletes whatever matches.
+
+=item * C<< XML::Bare::forcearray( [noderef] ) >>
+
+Turns the node reference into an array reference, whether that
+node is just a single node, or is already an array reference.
+
+=item * C<< XML::Bare::new_node( ... ) >>
+
+Creates a new node...
+
+=item * C<< XML::Bare::newhash( ... ) >>
+
+Creates a new hash with the specified value.
+
+=item * C<< XML::Bare::simplify( [noderef] ) >>
+
+Take a node with children that have immediate values and
+creates a hashref to reference those values by the name of
+each child.
+
+=back
+
+=head2 Functions Used Internally
+
+=over 2
+
+=item * C<< XML::Bare::c_parse() >>
+
+=item * C<< XML::Bare::c_parsefile() >>
+
+=item * C<< XML::Bare::free_tree() >>
+
+=item * C<< XML::Bare::xml2obj() >>
+
+=item * C<< XML::Bare::xml2obj_simple() >>
+
+=item * C<< XML::Bare::obj2xml() >>
 
 =back
 
@@ -762,36 +858,58 @@ Example:
 
 In comparison to other available perl xml parsers that create trees, XML::Bare
 is extremely fast. In order to measure the performance of loading and parsing
-compared to the alternatives, a test script has been created and is included
-with the distribution as 'test.pl'.
+compared to the alternatives, a templated speed comparison mechanism has been
+created and included with XML::Bare.
 
-The test script can compare the speed of XML::Bare against the following
-alternatives:
+The include makebench.pl file runs when you make the module and creates perl
+files within the bench directory corresponding to the .tmpl contained there.
+
+Currently there are three types of modules that can be tested against,
+executable parsers ( exe.tmpl ), tree parsers ( tree.tmpl ), and parsers
+that do not generated trees ( notree.tmpl ).
+
+A full list of modules currently tested against is as follows:
 
 =over 2
 
+=item * Tiny XML (exe)
+
+=item * EzXML (exe)
+
+=item * XMLIO (exe)
+
+=item * XML::LibXML (notree)
+
+=item * XML::Parser (notree)
+
+=item * XML::Parser::Expat (notree)
+
+=item * XML::Descent (notree)
+
 =item * XML::Parser::EasyTree
 
 =item * XML::Handler::Trees
 
 =item * XML::Twig
 
-=item * XML::LibXML
-
-Note that basic LibXML is included in the comparison, despite the
-fact that it does not create a tree.
-
 =item * XML::Smart
 
 =item * XML::Simple
 
+=item * XML::TreePP
+
+=item * XML::Trivial
+
+=item * XML::SAX::Simple
+
+=item * XML::Grove::Builder
+
+=item * XML::XPath::XMLParser
+
 =back
 
-To run the comparison, you must provide a number, 1-12, as a paramter
-to the script in order to choose which module to compare against. The
-script works this way because some of the modules have parts used by
-the other modules, which hides the loading time for the module tested
-later...
+To run the comparisons, run the appropriate perl file within the
+bench directory. (exe.pl, tree.pl, or notree.pl )
 
 The script measures the milliseconds of loading and parsing, and
 compares the time against the time of XML::Bare. So a 7 means
@@ -800,52 +918,65 @@ it takes 7 times as long as XML::Bare.
 Here is a combined table of the script run against each alternative
 using the included test.xml:
 
-  -Module-              load     parse    total
-  XML::Bare             1        1        1
-  XML::Parser::EasyTree 5.6811   29.2881  8.5366
-  XML::Handler::Trees   7.8083   30.1434  10.503
-  XML::Twig             31.0709  60.7735  34.5892
-  XML::LibXML (no tree) 13.1591  1.8211   11.7857
-  XML::Smart            6.9198   93.2242  17.1124
-  XML::Simple           3.4242   207.0007 29.5704
-  XML::SAX::Simple      9.82     191.0584 31.1326
-  XML::Trivial          5.8321   7.009    6.3731
-  XML::TreePP           2.5766   35.0588  6.4429
-  XML::XPath::XMLParser 12.4321  41.0182  16.651
-  XML::DOM::Lite        16.3544  14.8667  16.1905
-  TinyXML                                 4.2033
+  -Module-                   load     parse    total
+  XML::Bare                  1        1        1
+  XML::TreePP                2.3063   33.1776  6.1598
+  XML::Parser::EasyTree      4.9405   25.7278  7.4571
+  XML::Handler::Trees        7.2303   26.5688  9.6447
+  XML::Trivial               5.0636   12.4715  7.3046
+  XML::Smart                 6.8138   78.7939  15.8296
+  XML::Simple                2.7115   195.9411 26.5704
+  XML::SAX::Simple           8.7792   170.7313 28.3634
+  XML::Twig                  27.8266  56.4476  31.3594
+  XML::Grove::Builder        7.1267   26.1672  9.4064
+  XML::XPath::XMLParser      9.7783   35.5486  13.0002
+  XML::LibXML (notree)       11.0038  4.5758   10.6881
+  XML::Parser (notree)       4.4698   17.6448  5.8609
+  XML::Parser::Expat(notree) 3.7681   50.0382  6.0069
+  XML::Descent (notree)      6.0525   37.0265  11.0322
+  Tiny XML (exe)                               1.0095
+  EzXML (exe)                                  1.1284
+  XMLIO (exe)                                  1.0165
 
 Here is a combined table of the script run against each alternative
 using the included feed2.xml:
 
-  -Module-              load     parse    total
-  XML::Bare             1        1        1
-  XML::Parser::EasyTree 5.442    23.234   10.0313
-  XML::Handler::Trees   5.9811   20.5755  9.6939
-  XML::Twig             32.0006  44.811   35.3799
-  XML::LibXML (no tree) 13.5665  1.2518   10.0492
-  XML::Smart            6.8234   42.8422  16.2711
-  XML::Simple           3.9487   111.1732 31.6937
-  XML::SAX::Simple      10.1525  90.7282  32.7888
-  XML::Trivial          5.7941   28.5549  11.7381
-  XML::TreePP           2.8155   4.5963   3.9556
-  XML::XPath::XMLParser 11.9291  63.184   26.5266
-  XML::DOM::Lite        17.1702  13.8642  16.286
-  TinyXML                                 6.5016
+  -Module-                   load     parse    total
+  XML::Bare                  1        1        1
+  XML::TreePP                2.3068   23.7554  7.6921
+  XML::Parser::EasyTree      4.8799   25.3691  9.6257
+  XML::Handler::Trees        6.8545   33.1007  13.0575
+  XML::Trivial               5.0105   32.0043  11.4113
+  XML::Smart                 6.8489   45.4236  16.2809
+  XML::Simple                2.7168   90.7203  26.7525
+  XML::SAX::Simple           8.7386   94.8276  29.2166
+  XML::Twig                  28.3206  48.1014  33.1222
+  XML::Grove::Builder        7.2021   30.7926  12.9334
+  XML::XPath::XMLParser      9.6869   43.5032  17.4941
+  XML::LibXML (notree)       11.0023  5.022    10.5214
+  XML::Parser (notree)       4.3748   25.0213  5.9803
+  XML::Parser::Expat(notree) 3.6555   51.6426  7.4316
+  XML::Descent (notree)      5.9206   155.0289 18.7767
+  Tiny XML (exe)                               1.2212
+  EzXML (exe)                                  1.3618
+  XMLIO (exe)                                  1.0145
 
-These results show that XML::Bare is, at least on the test machine,
-~3-30 times faster loading and ~10-150 times faster parsing than
-any of the alternative tree parsers.
+These results show that XML::Bare is, at least on the
+test machine, running all tests within cygwin, faster
+at loading and parsing than everything being tested
+against.
 
-The following are shown as well:
+The following things are shown as well:
   - XML::Bare can parse XML and create a hash tree
   in less time than it takes LibXML just to parse.
   - XML::Bare can parse XML and create a hash tree
-  in 1/4 the time it takes TinyXML just to parse
+  in less time than all three binary parsers take
+  just to parse.
 
-Note that TinyXML is not a perl module and is timed
-by a dummy program that just uses the library to
-load and parse the example files.
+Note that the executable parsers are not perl modules
+and are timed using dummy programs that just uses the
+library to load and parse the example files. The files
+created to do such testing are available upon request.
 
 =head1 LICENSE
 
